/*
 * test_node_fragment.c
 *
 * Copyright 2022 Jonathan Schöbel <jonathan@Ubermos-2019>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 *
 */


#include <check.h>
#include <stdbool.h>
#include <stdlib.h>

#include "macro.h"
#include "status.h"

#include "node_fragment.c"


START_TEST(test_node_fragment)
{
	struct SH_Status status;
	struct SH_Fragment * fragment;
	struct SH_Data * data;
	const char * tag = "tag";

	data = SH_Data_new (NULL);

	/* valid tag - no error */
	fragment = SH_NodeFragment_new (tag, data, NULL);
	ck_assert_int_ne ((long int) fragment, (long int) NULL);

	ck_assert_ptr_eq (NULL, fragment->parent);
	ck_assert_str_eq (((struct SH_NodeFragment *) fragment)->tag, tag);

	SH_Fragment_free (fragment);

	/* valid tag - error */
	_status_preinit (status);
	fragment = SH_NodeFragment_new (tag, data, &status);
	ck_assert_int_ne ((long int) fragment, (long int) NULL);
	ck_assert_int_eq (status.status, SUCCESS);

	ck_assert_ptr_eq (NULL, fragment->parent);
	ck_assert_str_eq (((struct SH_NodeFragment *) fragment)->tag, tag);

	SH_Fragment_free (fragment);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_raw_new)
{
	struct SH_Status status;
	struct SH_Fragment * fragment;
	struct SH_Data * data;
	char * tag1 = strdup ("tag");
	char * tag2 = strdup ("tag");

	data = SH_Data_new (NULL);

	/* valid tag - no error */
	fragment = SH_NodeFragment_raw_new (tag1, data, NULL);
	ck_assert_int_ne ((long int) fragment, (long int) NULL);

	ck_assert_ptr_eq (NULL, fragment->parent);
	ck_assert_ptr_eq (((struct SH_NodeFragment *) fragment)->tag, tag1);

	SH_Fragment_free (fragment);

	/* valid tag - error */
	_status_preinit (status);
	fragment = SH_NodeFragment_raw_new (tag2, data, &status);
	ck_assert_int_ne ((long int) fragment, (long int) NULL);
	ck_assert_int_eq (status.status, SUCCESS);

	ck_assert_ptr_eq (NULL, fragment->parent);
	ck_assert_ptr_eq (((struct SH_NodeFragment *) fragment)->tag, tag2);

	SH_Fragment_free (fragment);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_copy)
{
	struct SH_Status status;
	struct SH_Fragment * fragment;
	struct SH_Fragment * copy;
	struct SH_Data * data;

	data = SH_Data_new (NULL);

	fragment = SH_NodeFragment_new ("html", data, NULL);

	/* without error */
	copy = SH_NodeFragment_copy (((struct SH_NodeFragment *) fragment),
				     NULL);

	ck_assert_ptr_ne (copy, NULL);
	ck_assert_ptr_ne (fragment, copy);
	ck_assert_ptr_eq (NULL, copy->parent);
	ck_assert_str_eq (((struct SH_NodeFragment *) fragment)->tag,
			  ((struct SH_NodeFragment *) copy)->tag);
	ck_assert_int_eq (((struct SH_NodeFragment *) copy)->child_n, 0);
	ck_assert_ptr_ne (((struct SH_NodeFragment *) fragment)->childs,
			  ((struct SH_NodeFragment *) copy)->childs);

	SH_Fragment_free (copy);

	/* with error */
	_status_preinit (status);
	copy = SH_NodeFragment_copy (((struct SH_NodeFragment *) fragment),
				     &status);
	ck_assert_int_eq (status.status, SUCCESS);

	ck_assert_ptr_ne (copy, NULL);
	ck_assert_ptr_ne (fragment, copy);
	ck_assert_ptr_eq (NULL, copy->parent);
	ck_assert_str_eq (((struct SH_NodeFragment *) fragment)->tag,
			  ((struct SH_NodeFragment *) copy)->tag);
	ck_assert_int_eq (((struct SH_NodeFragment *) copy)->child_n, 0);
	ck_assert_ptr_ne (((struct SH_NodeFragment *) fragment)->childs,
			  ((struct SH_NodeFragment *) copy)->childs);

	SH_Fragment_free (fragment);
	SH_Fragment_free (copy);

	SH_Data_free (data);
}
END_TEST

static void
check_childs (struct SH_NodeFragment * fragment, struct SH_NodeFragment * copy)
{
	size_t index;

	for (index = 0; index < copy->child_n; index++)
	{
		ck_assert_ptr_ne (fragment->childs[index],
				  copy->childs[index]);
		ck_assert_ptr_eq (copy, get_parent (copy->childs[index]));
		ck_assert_str_eq (((struct SH_NodeFragment *) fragment->childs[index])->tag,
				  ((struct SH_NodeFragment *) copy->childs[index])->tag);
		ck_assert_int_eq (((struct SH_NodeFragment *) fragment->childs[index])->child_n,
				  ((struct SH_NodeFragment *) copy->childs[index])->child_n);

		check_childs (((struct SH_NodeFragment *) fragment->childs[index]),
			      ((struct SH_NodeFragment *) copy->childs[index]));
	}

	return;
}

START_TEST(test_node_fragment_deepcopy)
{
	struct SH_Status status;
	struct SH_Fragment * fragment;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * copy;
	struct SH_Data * data;

	data = SH_Data_new (NULL);

	fragment = SH_NodeFragment_new ("html", data, NULL);

	child1 = SH_NodeFragment_new ("head", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) fragment),
				      child1, NULL);

	child2 = SH_NodeFragment_new ("title", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) child1),
				      child2, NULL);


	child1 = SH_NodeFragment_new ("body", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) fragment),
				      child1, NULL);

	child2 = SH_NodeFragment_new ("header", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) child1),
				      child2, NULL);

	child2 = SH_NodeFragment_new ("main", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) child1),
				      child2, NULL);

	child2 = SH_NodeFragment_new ("footer", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) child1),
				      child2, NULL);


	/* without error */
	copy = SH_Fragment_copy (fragment, NULL);

	ck_assert_ptr_ne (copy, NULL);
	ck_assert_ptr_ne (fragment, copy);
	ck_assert_ptr_eq (NULL, copy->parent);
	ck_assert_str_eq (((struct SH_NodeFragment *) fragment)->tag,
			  ((struct SH_NodeFragment *) copy)->tag);
	ck_assert_int_eq (((struct SH_NodeFragment *) fragment)->child_n,
			  ((struct SH_NodeFragment *) copy)->child_n);
	ck_assert_ptr_ne (((struct SH_NodeFragment *) fragment)->childs,
			  ((struct SH_NodeFragment *) copy)->childs);

	check_childs (((struct SH_NodeFragment *) fragment),
		      ((struct SH_NodeFragment *) copy));

	SH_Fragment_free (copy);

	/* with error */
	_status_preinit (status);
	copy = SH_Fragment_copy (fragment, &status);
	ck_assert_int_eq (status.status, SUCCESS);

	ck_assert_ptr_ne (copy, NULL);
	ck_assert_ptr_ne (fragment, copy);
	ck_assert_ptr_eq (NULL, copy->parent);
	ck_assert_str_eq (((struct SH_NodeFragment *) fragment)->tag,
			  ((struct SH_NodeFragment *) copy)->tag);
	ck_assert_int_eq (((struct SH_NodeFragment *) fragment)->child_n,
			  ((struct SH_NodeFragment *) copy)->child_n);
	ck_assert_ptr_ne (((struct SH_NodeFragment *) fragment)->childs,
			  ((struct SH_NodeFragment *) copy)->childs);

	check_childs (((struct SH_NodeFragment *) fragment),
		      ((struct SH_NodeFragment *) copy));

	SH_Fragment_free (fragment);
	SH_Fragment_free (copy);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_tag)
{
	struct SH_Status status;
	struct SH_Fragment * fragment;
	struct SH_Data * data;
	char * tag;
	const char * tag1 = "html";
	const char * tag2 = "body";

	data = SH_Data_new (NULL);

	/* no error */
	fragment = SH_NodeFragment_new (tag1, data, NULL);

	tag = SH_NodeFragment_raw_get_tag (((struct SH_NodeFragment *) fragment));
	ck_assert_str_eq (tag, tag1);

	tag = SH_NodeFragment_get_tag (((struct SH_NodeFragment *) fragment),
				       NULL);
	ck_assert_str_eq (tag, tag1);
	free (tag);

	SH_Fragment_free (fragment);

	/* error */
	fragment = SH_NodeFragment_new (tag2, data, NULL);

	_status_preinit (status);
	tag = SH_NodeFragment_get_tag (((struct SH_NodeFragment *) fragment),
				       &status);
	ck_assert_str_eq (tag, tag2);
	ck_assert_int_eq (status.status, SUCCESS);
	free (tag);

	SH_Fragment_free (fragment);
	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_alloc)
{
	ck_assert_int_eq (0, get_child_alloc_size (0));
	ck_assert_int_eq (CHILD_CHUNK, get_child_alloc_size (1));
	ck_assert_int_eq (CHILD_CHUNK, get_child_alloc_size (CHILD_CHUNK));
	ck_assert_int_eq (2*CHILD_CHUNK, get_child_alloc_size (CHILD_CHUNK+1));
	ck_assert_int_eq (SIZE_MAX, get_child_alloc_size (SIZE_MAX));
	for (size_t i = 0; i < CHILD_CHUNK; i++)
	{
		ck_assert_int_le (SIZE_MAX-i, get_child_alloc_size (SIZE_MAX-i));
	}
}
END_TEST

START_TEST(test_node_fragment_child)
{
	struct SH_Status status;
	struct SH_Fragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Data * data;
	bool boolean;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child1 = SH_NodeFragment_new ("head", data, NULL);
	child2 = SH_NodeFragment_new ("body", data, NULL);

	/* no error */
	ck_assert_int_eq (((struct SH_NodeFragment *) parent)->child_n, 0);

	boolean = SH_NodeFragment_append_child (((struct SH_NodeFragment *) parent),
						child1, NULL);
	ck_assert_int_eq (boolean, TRUE);
	ck_assert_int_eq (((struct SH_NodeFragment *) parent)->child_n,
			  1);
	ck_assert_ptr_eq (((struct SH_NodeFragment *) parent)->childs[0],
			  child1);

	ck_assert_ptr_eq (parent, get_parent (child1));

	/* with error */
	ck_assert_int_eq (((struct SH_NodeFragment *) parent)->child_n, 1);

	_status_preinit (status);
	boolean = SH_NodeFragment_append_child (((struct SH_NodeFragment *) parent),
						child2, &status);
	ck_assert_int_eq (boolean, TRUE);
	ck_assert_int_eq (status.status, SUCCESS);
	ck_assert_int_eq (((struct SH_NodeFragment *) parent)->child_n,
			  2);
	ck_assert_ptr_eq (((struct SH_NodeFragment *) parent)->childs[1],
			  child2);

	ck_assert_ptr_eq (parent, get_parent (child2));

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_get_child)
{
	struct SH_Status status;
	struct SH_Fragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Data * data;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child1 = SH_NodeFragment_new ("body", data, NULL);

	SH_NodeFragment_append_child (((struct SH_NodeFragment *) parent),
				      child1, NULL);

	/* without error */
	child2 = SH_NodeFragment_get_child (((struct SH_NodeFragment *) parent),
					    0, NULL);
	ck_assert_ptr_eq (child1, child2);

	child2 = SH_NodeFragment_get_child (((struct SH_NodeFragment *) parent),
					    1, NULL);
	ck_assert_ptr_eq (NULL, child2);

	/* with error */
	_status_preinit (status);
	child2 = SH_NodeFragment_get_child (((struct SH_NodeFragment *) parent),
					    0, &status);
	ck_assert_ptr_eq (child1, child2);
	ck_assert_int_eq (status.status, SUCCESS);

	_status_preinit (status);
	child2 = SH_NodeFragment_get_child (((struct SH_NodeFragment *) parent),
					    1, &status);
	ck_assert_ptr_eq (NULL, child2);
	ck_assert_int_eq (status.status, E_VALUE);

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_is_child)
{
	struct SH_Fragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Data * data;
	bool boolean;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child1 = SH_NodeFragment_new ("head", data, NULL);
	child2 = SH_NodeFragment_new ("title", data, NULL);

	SH_NodeFragment_append_child (((struct SH_NodeFragment *) parent),
				      child1, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) child1),
				      child2, NULL);

	boolean = SH_NodeFragment_is_child (((struct SH_NodeFragment *) parent),
					    child1);
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_child (((struct SH_NodeFragment *) parent),
					    child2);
	ck_assert_int_eq (boolean, FALSE);

	boolean = SH_NodeFragment_is_child (((struct SH_NodeFragment *) child1),
					    child2);
	ck_assert_int_eq (boolean, TRUE);

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_is_descendant)
{
	struct SH_Fragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	struct SH_Data * data;
	bool boolean;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child1 = SH_NodeFragment_new ("head", data, NULL);
	child2 = SH_NodeFragment_new ("body", data, NULL);
	child3 = SH_NodeFragment_new ("title", data, NULL);

	SH_NodeFragment_append_child (((struct SH_NodeFragment *) parent),
				       child1, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) parent),
				       child2, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) child1),
				       child3, NULL);

	boolean = SH_NodeFragment_is_descendant (((struct SH_NodeFragment *) parent),
						  child1);
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_descendant (((struct SH_NodeFragment *) parent),
						  child2);
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_descendant (((struct SH_NodeFragment *) parent),
						  child3);
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_descendant (((struct SH_NodeFragment *) child1),
						  child2);
	ck_assert_int_eq (boolean, FALSE);

	boolean = SH_NodeFragment_is_descendant (((struct SH_NodeFragment *) child1),
						  child3);
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_descendant (((struct SH_NodeFragment *) child2),
						  child3);
	ck_assert_int_eq (boolean, FALSE);

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_is_parent)
{
	struct SH_Fragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Data * data;
	bool boolean;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child1 = SH_NodeFragment_new ("head", data, NULL);
	child2 = SH_NodeFragment_new ("title", data, NULL);

	SH_NodeFragment_append_child (((struct SH_NodeFragment *) parent),
				      child1, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) child1),
				      child2, NULL);

	boolean = SH_NodeFragment_is_parent (child1,
	                                     (struct SH_NodeFragment *)
	                                     parent);
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_parent (child2,
	                                     (struct SH_NodeFragment *)
	                                     parent);
	ck_assert_int_eq (boolean, FALSE);

	boolean = SH_NodeFragment_is_parent (child2,
	                                     (struct SH_NodeFragment *)
	                                     child1);
	ck_assert_int_eq (boolean, TRUE);

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_is_ancestor)
{
	struct SH_Fragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	struct SH_Data * data;
	bool boolean;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child1 = SH_NodeFragment_new ("head", data, NULL);
	child2 = SH_NodeFragment_new ("body", data, NULL);
	child3 = SH_NodeFragment_new ("title", data, NULL);

	SH_NodeFragment_append_child (((struct SH_NodeFragment *)parent),
				       child1, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *)parent),
				       child2, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *)child1),
				       child3, NULL);

	boolean = SH_NodeFragment_is_ancestor (child1,
	                                       ((struct SH_NodeFragment *)
	                                       parent));
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_ancestor (child2,
	                                       ((struct SH_NodeFragment *)
	                                       parent));
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_ancestor (child3,
	                                       ((struct SH_NodeFragment *)
	                                       parent));
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_ancestor (child2,
	                                       ((struct SH_NodeFragment *)
	                                       child1));
	ck_assert_int_eq (boolean, FALSE);

	boolean = SH_NodeFragment_is_ancestor (child3,
	                                       ((struct SH_NodeFragment *)
	                                       child1));
	ck_assert_int_eq (boolean, TRUE);

	boolean = SH_NodeFragment_is_ancestor (child3,
	                                       ((struct SH_NodeFragment *)
	                                       child2));
	ck_assert_int_eq (boolean, FALSE);

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_insert_no_error)
{
	SH_Data * data;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)
	         SH_NodeFragment_new ("body", data, NULL);
	child1 = SH_NodeFragment_new ("header", data, NULL);
	child2 = SH_NodeFragment_new ("main", data, NULL);
	child3 = SH_NodeFragment_new ("footer", data, NULL);

	/* insert - success */
	result = insert_child (parent, child1, 0, NULL);
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (1, parent->child_n);
	ck_assert_int_le (1, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child1, parent->childs[0]);
	ck_assert_ptr_eq (parent, child1->parent);

	/* reinsert - failing */
	result = insert_child (parent, child1, 0, NULL);
	ck_assert_int_eq (FALSE, result);

	/* insert copy - success */
	child = SH_Fragment_copy (child1, NULL);
	result = insert_child (parent, child, 0, NULL);
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (2, parent->child_n);
	ck_assert_int_le (2, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child, parent->childs[0]);
	ck_assert_ptr_eq (child1, parent->childs[1]);
	ck_assert_ptr_eq (parent, child->parent);
	ck_assert_ptr_eq (parent, child1->parent);

	/* insert parent into child - failing */
	result = insert_child ((struct SH_NodeFragment *)child1,
	                       (struct SH_Fragment *)parent,
	                       0, NULL);
	ck_assert_int_eq (FALSE, result);

	/* insert inbetween */
	result = insert_child (parent, child2, 1, NULL);
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (3, parent->child_n);
	ck_assert_int_le (3, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child, parent->childs[0]);
	ck_assert_ptr_eq (child1, parent->childs[2]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (parent, child->parent);
	ck_assert_ptr_eq (parent, child1->parent);
	ck_assert_ptr_eq (parent, child2->parent);

	/* insert after */
	result = insert_child (parent, child3, 3, NULL);
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (4, parent->child_n);
	ck_assert_int_le (4, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child, parent->childs[0]);
	ck_assert_ptr_eq (child1, parent->childs[2]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (child3, parent->childs[3]);
	ck_assert_ptr_eq (parent, child->parent);
	ck_assert_ptr_eq (parent, child1->parent);
	ck_assert_ptr_eq (parent, child2->parent);
	ck_assert_ptr_eq (parent, child3->parent);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_insert_with_error)
{
	struct SH_Status status;
	SH_Data * data;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)
	         SH_NodeFragment_new ("body", data, NULL);
	child1 = SH_NodeFragment_new ("header", data, NULL);
	child2 = SH_NodeFragment_new ("main", data, NULL);
	child3 = SH_NodeFragment_new ("footer", data, NULL);

	/* insert - success */
	_status_preinit (status);
	result = insert_child (parent, child1, 0, &status);
	ck_assert (succeed (&status));
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (1, parent->child_n);
	ck_assert_int_le (1, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child1, parent->childs[0]);
	ck_assert_ptr_eq (parent, child1->parent);

	/* reinsert - failing */
	_status_preinit (status);
	result = insert_child (parent, child1, 0, &status);
	ck_assert (failed (&status));
	ck_assert_int_eq (FALSE, result);
	ck_assert_int_eq (E_STATE, status.status);

	/* insert copy - success */
	child = SH_Fragment_copy (child1, NULL);
	_status_preinit (status);
	result = insert_child (parent, child, 0, &status);
	ck_assert (succeed (&status));
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (2, parent->child_n);
	ck_assert_int_le (2, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child, parent->childs[0]);
	ck_assert_ptr_eq (child1, parent->childs[1]);
	ck_assert_ptr_eq (parent, child->parent);
	ck_assert_ptr_eq (parent, child1->parent);

	/* insert parent into child - failing */
	_status_preinit (status);
	result = insert_child ((struct SH_NodeFragment *)child1,
	                       (struct SH_Fragment *)parent,
	                       0, &status);
	ck_assert (failed (&status));
	ck_assert_int_eq (FALSE, result);
	ck_assert_int_eq (E_STATE, status.status);

	/* insert inbetween */
	_status_preinit (status);
	result = insert_child (parent, child2, 1, &status);
	ck_assert (succeed (&status));
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (3, parent->child_n);
	ck_assert_int_le (3, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child, parent->childs[0]);
	ck_assert_ptr_eq (child1, parent->childs[2]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (parent, child->parent);
	ck_assert_ptr_eq (parent, child1->parent);
	ck_assert_ptr_eq (parent, child2->parent);

	/* insert after */
	_status_preinit (status);
	result = insert_child (parent, child3, 3, &status);
	ck_assert (succeed (&status));
	ck_assert_int_eq (TRUE, result);
	ck_assert_int_eq (4, parent->child_n);
	ck_assert_int_le (4, parent->child_s);
	ck_assert_int_eq (CHILD_CHUNK, parent->child_s);
	ck_assert_ptr_eq (child, parent->childs[0]);
	ck_assert_ptr_eq (child1, parent->childs[2]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (child3, parent->childs[3]);
	ck_assert_ptr_eq (parent, child->parent);
	ck_assert_ptr_eq (parent, child1->parent);
	ck_assert_ptr_eq (parent, child2->parent);
	ck_assert_ptr_eq (parent, child3->parent);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_insert_insert_no_error)
{
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("body",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("header", data, NULL);
	child2 = SH_NodeFragment_new ("main", data, NULL);
	child3 = SH_NodeFragment_new ("footer", data, NULL);

	/* out of range - fail */
	result = SH_NodeFragment_insert_child (parent, child1, 1, NULL);
	ck_assert_int_eq (FALSE, result);

	/* insert - success */
	result = SH_NodeFragment_insert_child (parent, child2, 0, NULL);
	ck_assert_int_eq (TRUE, result);

	/* insert before - success */
	result = SH_NodeFragment_insert_child (parent, child1, 0, NULL);
	ck_assert_int_eq (TRUE, result);

	/* insert after - success */
	result = SH_NodeFragment_insert_child (parent, child3, 2, NULL);
	ck_assert_int_eq (TRUE, result);

	ck_assert_int_eq (3, parent->child_n);
	ck_assert_ptr_eq (child1, parent->childs[0]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (child3, parent->childs[2]);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_insert_insert_with_error)
{
	struct SH_Status status;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("body",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("header", data, NULL);
	child2 = SH_NodeFragment_new ("main", data, NULL);
	child3 = SH_NodeFragment_new ("footer", data, NULL);

	/* out of range - fail */
	_status_preinit (status);
	result = SH_NodeFragment_insert_child (parent, child1, 1,
	                                       &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert_int_eq (E_VALUE, status.status);
	ck_assert (failed (&status));

	/* insert - success */
	_status_preinit (status);
	result = SH_NodeFragment_insert_child (parent, child2, 0,
	                                       &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	/* insert before - success */
	_status_preinit (status);
	result = SH_NodeFragment_insert_child (parent, child1, 0,
	                                       &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	/* insert after - success */
	_status_preinit (status);
	result = SH_NodeFragment_insert_child (parent, child3, 2,
	                                       &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	ck_assert_int_eq (3, parent->child_n);
	ck_assert_ptr_eq (child1, parent->childs[0]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (child3, parent->childs[2]);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_insert_relative_no_error)
{
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("body",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("header", data, NULL);
	child2 = SH_NodeFragment_new ("main", data, NULL);
	child3 = SH_NodeFragment_new ("footer", data, NULL);

	/* test erroneous call */
	result = SH_NodeFragment_insert_child_before (child2, child1,
	                                              NULL);
	ck_assert_int_eq (FALSE, result);

	result = SH_NodeFragment_insert_child_after (child2, child3,
	                                             NULL);
	ck_assert_int_eq (FALSE, result);

	/* test bug check */
	child2->parent = parent;

	result = SH_NodeFragment_insert_child_before (child2, child1,
	                                              NULL);
	ck_assert_int_eq (FALSE, result);

	result = SH_NodeFragment_insert_child_after (child2, child3,
	                                             NULL);
	ck_assert_int_eq (FALSE, result);

	child2->parent = NULL;

	/* test real */
	result = SH_NodeFragment_insert_child (parent, child2, 0,
	                                       NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_insert_child_before (child2, child1,
	                                              NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_insert_child_after (child2, child3,
	                                             NULL);
	ck_assert_int_eq (TRUE, result);

	ck_assert_int_eq (3, parent->child_n);
	ck_assert_ptr_eq (child1, parent->childs[0]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (child3, parent->childs[2]);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_insert_relative_with_error)
{
	struct SH_Status status;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("body",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("header", data, NULL);
	child2 = SH_NodeFragment_new ("main", data, NULL);
	child3 = SH_NodeFragment_new ("footer", data, NULL);

	/* test erroneous call */
	_status_preinit (status);
	result = SH_NodeFragment_insert_child_before (child2, child1,
	                                              &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert_int_eq (E_STATE, status.status);
	ck_assert (failed (&status));

	_status_preinit (status);
	result = SH_NodeFragment_insert_child_after (child2, child3,
	                                             &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert_int_eq (E_STATE, status.status);
	ck_assert (failed (&status));

	/* test bug check */
	child2->parent = parent;

	_status_preinit (status);
	result = SH_NodeFragment_insert_child_before (child2, child1,
	                                              &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert_int_eq (E_BUG, status.status);
	ck_assert (failed (&status));

	_status_preinit (status);
	result = SH_NodeFragment_insert_child_after (child2, child3,
	                                             &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert_int_eq (E_BUG, status.status);
	ck_assert (failed (&status));

	child2->parent = NULL;

	/* test real */
	_status_preinit (status);
	result = SH_NodeFragment_insert_child (parent, child2, 0,
	                                       &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	_status_preinit (status);
	result = SH_NodeFragment_insert_child_before (child2, child1,
	                                              &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	_status_preinit (status);
	result = SH_NodeFragment_insert_child_after (child2, child3,
	                                             &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	ck_assert_int_eq (3, parent->child_n);
	ck_assert_ptr_eq (child1, parent->childs[0]);
	ck_assert_ptr_eq (child2, parent->childs[1]);
	ck_assert_ptr_eq (child3, parent->childs[2]);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_no_error)
{
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	struct SH_Fragment * child4;
	struct SH_Fragment * child5;
	struct SH_Fragment * child6;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("body",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("p", data, NULL);
	child2 = SH_NodeFragment_new ("p", data, NULL);
	child3 = SH_NodeFragment_new ("p", data, NULL);
	child4 = SH_NodeFragment_new ("p", data, NULL);
	child5 = SH_NodeFragment_new ("p", data, NULL);
	child6 = SH_NodeFragment_new ("p", data, NULL);

	result = SH_NodeFragment_append_child (parent, child1, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child2, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child3, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child4, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child5, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child6, NULL);
	ck_assert_int_eq (TRUE, result);


	ck_assert_int_eq (10, parent->child_s);
	ck_assert_int_eq (6, parent->child_n);
	ck_assert_ptr_eq (child1, parent->childs[0]);

	child = remove_child (parent, 0, NULL);
	ck_assert_ptr_eq (child, child1);
	ck_assert_ptr_eq (NULL, child->parent);
	SH_Fragment_free (child1);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (5, parent->child_n);
	ck_assert_ptr_eq (child2, parent->childs[0]);

	child = remove_child (parent, 0, NULL);
	ck_assert_ptr_eq (child, child2);
	ck_assert_ptr_eq (NULL, child->parent);
	SH_Fragment_free (child2);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (4, parent->child_n);
	ck_assert_ptr_eq (child3, parent->childs[0]);

	child = remove_child (parent, 0, NULL);
	ck_assert_ptr_eq (child, child3);
	ck_assert_ptr_eq (NULL, child->parent);
	SH_Fragment_free (child3);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (3, parent->child_n);
	ck_assert_ptr_eq (child4, parent->childs[0]);

	child = remove_child (parent, 0, NULL);
	ck_assert_ptr_eq (child, child4);
	ck_assert_ptr_eq (NULL, child->parent);
	SH_Fragment_free (child4);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (2, parent->child_n);
	ck_assert_ptr_eq (child5, parent->childs[0]);

	child = remove_child (parent, 0, NULL);
	ck_assert_ptr_eq (child, child5);
	ck_assert_ptr_eq (NULL, child->parent);
	SH_Fragment_free (child5);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (1, parent->child_n);
	ck_assert_ptr_eq (child6, parent->childs[0]);

	child = remove_child (parent, 0, NULL);
	ck_assert_ptr_eq (child, child6);
	ck_assert_ptr_eq (NULL, child->parent);
	SH_Fragment_free (child6);
	ck_assert_int_eq (0, parent->child_s);
	ck_assert_int_eq (0, parent->child_n);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_with_error)
{
	struct SH_Status status;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	struct SH_Fragment * child1;
	struct SH_Fragment * child2;
	struct SH_Fragment * child3;
	struct SH_Fragment * child4;
	struct SH_Fragment * child5;
	struct SH_Fragment * child6;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("body",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("p", data, NULL);
	child2 = SH_NodeFragment_new ("p", data, NULL);
	child3 = SH_NodeFragment_new ("p", data, NULL);
	child4 = SH_NodeFragment_new ("p", data, NULL);
	child5 = SH_NodeFragment_new ("p", data, NULL);
	child6 = SH_NodeFragment_new ("p", data, NULL);

	result = SH_NodeFragment_append_child (parent, child1, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child2, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child3, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child4, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child5, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_append_child (parent, child6, NULL);
	ck_assert_int_eq (TRUE, result);


	ck_assert_int_eq (10, parent->child_s);
	ck_assert_int_eq (6, parent->child_n);
	ck_assert_ptr_eq (child1, parent->childs[0]);

	_status_preinit (status);
	child = remove_child (parent, 0, &status);
	ck_assert_ptr_eq (child, child1);
	ck_assert_ptr_eq (NULL, child->parent);
	ck_assert (succeed (&status));
	SH_Fragment_free (child1);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (5, parent->child_n);
	ck_assert_ptr_eq (child2, parent->childs[0]);

	_status_preinit (status);
	child = remove_child (parent, 0, &status);
	ck_assert_ptr_eq (child, child2);
	ck_assert_ptr_eq (NULL, child->parent);
	ck_assert (succeed (&status));
	SH_Fragment_free (child2);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (4, parent->child_n);
	ck_assert_ptr_eq (child3, parent->childs[0]);

	_status_preinit (status);
	child = remove_child (parent, 0, &status);
	ck_assert_ptr_eq (child, child3);
	ck_assert_ptr_eq (NULL, child->parent);
	ck_assert (succeed (&status));
	SH_Fragment_free (child3);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (3, parent->child_n);
	ck_assert_ptr_eq (child4, parent->childs[0]);

	_status_preinit (status);
	child = remove_child (parent, 0, &status);
	ck_assert_ptr_eq (child, child4);
	ck_assert_ptr_eq (NULL, child->parent);
	ck_assert (succeed (&status));
	SH_Fragment_free (child4);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (2, parent->child_n);
	ck_assert_ptr_eq (child5, parent->childs[0]);

	_status_preinit (status);
	child = remove_child (parent, 0, &status);
	ck_assert_ptr_eq (child, child5);
	ck_assert_ptr_eq (NULL, child->parent);
	ck_assert (succeed (&status));
	SH_Fragment_free (child5);
	ck_assert_int_eq (5, parent->child_s);
	ck_assert_int_eq (1, parent->child_n);
	ck_assert_ptr_eq (child6, parent->childs[0]);

	_status_preinit (status);
	child = remove_child (parent, 0, &status);
	ck_assert_ptr_eq (child, child6);
	ck_assert_ptr_eq (NULL, child->parent);
	ck_assert (succeed (&status));
	SH_Fragment_free (child6);
	ck_assert_int_eq (0, parent->child_s);
	ck_assert_int_eq (0, parent->child_n);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_remove_self_no_error)
{
	struct SH_Fragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child = SH_NodeFragment_new ("head", data, NULL);

	result = SH_NodeFragment_append_child ((struct SH_NodeFragment *)
	                                       parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	/* test erroneous call */
	result = SH_Fragment_remove (parent, NULL);
	ck_assert_int_eq (FALSE, result);

	/* test real */
	result = SH_Fragment_remove (child, NULL);
	ck_assert_int_eq (TRUE, result);

	/* test bug check */
	child->parent = (struct SH_NodeFragment *)parent;
	result = SH_Fragment_remove (child, NULL);
	ck_assert_int_eq (FALSE, result);

	SH_Fragment_free (parent);
	SH_Fragment_free (child);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_remove_self_with_error)
{
	struct SH_Status status;
	struct SH_Fragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child = SH_NodeFragment_new ("head", data, NULL);

	result = SH_NodeFragment_append_child ((struct SH_NodeFragment *)
	                                       parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	/* test erroneous call */
	_status_preinit (status);
	result = SH_Fragment_remove (parent, &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert (failed (&status));
	ck_assert_int_eq (E_STATE, status.status);

	/* test real */
	_status_preinit (status);
	result = SH_Fragment_remove (child, &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	/* test bug check */
	child->parent = (struct SH_NodeFragment *)parent;
	_status_preinit (status);
	result = SH_Fragment_remove (child, &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert (failed (&status));
	ck_assert_int_eq (E_BUG, status.status);

	SH_Fragment_free (parent);
	SH_Fragment_free (child);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_delete_self_no_error)
{
	struct SH_Fragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child = SH_NodeFragment_new ("head", data, NULL);

	/* test erroneous call */
	result = SH_Fragment_delete (parent, NULL);
	ck_assert_int_eq (FALSE, result);

	/* test bug check */
	child->parent = (struct SH_NodeFragment *)parent;
	result = SH_Fragment_delete (child, NULL);
	ck_assert_int_eq (FALSE, result);
	child->parent = NULL;

	result = SH_NodeFragment_append_child ((struct SH_NodeFragment *)
	                                       parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	/* test real */
	result = SH_Fragment_delete (child, NULL);
	ck_assert_int_eq (TRUE, result);

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_delete_self_with_error)
{
	struct SH_Status status;
	struct SH_Fragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = SH_NodeFragment_new ("html", data, NULL);
	child = SH_NodeFragment_new ("head", data, NULL);

	/* test erroneous call */
	_status_preinit (status);
	result = SH_Fragment_remove (parent, &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert (failed (&status));
	ck_assert_int_eq (E_STATE, status.status);

	/* test bug check */
	child->parent = (struct SH_NodeFragment *)parent;
	_status_preinit (status);
	result = SH_Fragment_delete (child, &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert (failed (&status));
	ck_assert_int_eq (E_BUG, status.status);
	child->parent = NULL;

	result = SH_NodeFragment_append_child ((struct SH_NodeFragment *)
	                                       parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	/* test real */
	_status_preinit (status);
	result = SH_Fragment_delete (child, &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	SH_Fragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_remove_no_error)
{
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("html",
	                                                        data,
	                                                        NULL);
	child = SH_NodeFragment_new ("body", data, NULL);

	result = SH_NodeFragment_append_child (parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_remove_child (parent, 1, NULL);
	ck_assert_int_eq (FALSE, result);

	result = SH_NodeFragment_remove_child (parent, 0, NULL);
	ck_assert_int_eq (TRUE, result);

	SH_NodeFragment_free (parent);
	SH_Fragment_free (child);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_remove_with_error)
{
	struct SH_Status status;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("html",
	                                                        data,
	                                                        NULL);
	child = SH_NodeFragment_new ("body", data, NULL);

	result = SH_NodeFragment_append_child (parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	_status_preinit (status);
	result = SH_NodeFragment_remove_child (parent, 1, &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert (failed (&status));
	ck_assert_int_eq (E_VALUE, status.status);

	_status_preinit (status);
	result = SH_NodeFragment_remove_child (parent, 0, &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	SH_NodeFragment_free (parent);
	SH_Fragment_free (child);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_delete_no_error)
{
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("html",
	                                                        data,
	                                                        NULL);
	child = SH_NodeFragment_new ("body", data, NULL);

	result = SH_NodeFragment_append_child (parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	result = SH_NodeFragment_delete_child (parent, 1, NULL);
	ck_assert_int_eq (FALSE, result);

	result = SH_NodeFragment_delete_child (parent, 0, NULL);
	ck_assert_int_eq (TRUE, result);

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_delete_with_error)
{
	struct SH_Status status;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("html",
	                                                        data,
	                                                        NULL);
	child = SH_NodeFragment_new ("body", data, NULL);

	result = SH_NodeFragment_append_child (parent, child, NULL);
	ck_assert_int_eq (TRUE, result);

	_status_preinit (status);
	result = SH_NodeFragment_delete_child (parent, 1, &status);
	ck_assert_int_eq (FALSE, result);
	ck_assert (failed (&status));
	ck_assert_int_eq (E_VALUE, status.status);

	_status_preinit (status);
	result = SH_NodeFragment_delete_child (parent, 0, &status);
	ck_assert_int_eq (TRUE, result);
	ck_assert (succeed (&status));

	SH_NodeFragment_free (parent);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_pop_no_error)
{
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	struct SH_Fragment * child1;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("html",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("body", data, NULL);

	result = SH_NodeFragment_append_child (parent, child1, NULL);
	ck_assert_int_eq (TRUE, result);

	child = SH_NodeFragment_pop_child (parent, 1, NULL);
	ck_assert_ptr_eq (NULL, child);

	child = SH_NodeFragment_pop_child (parent, 0, NULL);
	ck_assert_ptr_eq (child, child1);

	SH_NodeFragment_free (parent);
	SH_Fragment_free (child);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_child_remove_pop_with_error)
{
	struct SH_Status status;
	struct SH_NodeFragment * parent;
	struct SH_Fragment * child;
	struct SH_Fragment * child1;
	SH_Data * data;
	bool result;

	data = SH_Data_new (NULL);

	parent = (struct SH_NodeFragment *)SH_NodeFragment_new ("html",
	                                                        data,
	                                                        NULL);
	child1 = SH_NodeFragment_new ("body", data, NULL);

	result = SH_NodeFragment_append_child (parent, child1, NULL);
	ck_assert_int_eq (TRUE, result);

	_status_preinit (status);
	child = SH_NodeFragment_pop_child (parent, 1, &status);
	ck_assert_ptr_eq (NULL, child);
	ck_assert (failed (&status));
	ck_assert_int_eq (E_VALUE, status.status);

	_status_preinit (status);
	child = SH_NodeFragment_pop_child (parent, 0, &status);
	ck_assert_ptr_eq (child, child1);
	ck_assert (succeed (&status));

	SH_NodeFragment_free (parent);
	SH_Fragment_free (child);

	SH_Data_free (data);
}
END_TEST

START_TEST(test_node_fragment_html)
{
	struct SH_Status status;
	struct SH_Fragment * fragment1;
	struct SH_Fragment * fragment2;
	struct SH_Data * data;
	struct SH_Text * text;
	char * string;
	size_t length;

	data = SH_Data_new (NULL);

	/* no error */
	fragment1 = SH_NodeFragment_new ("html", data, NULL);
	fragment2 = SH_NodeFragment_new ("body", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) fragment1),
				      fragment2, NULL);

	text = SH_Fragment_to_html (fragment1, INLINE, 0, 1, INDENT_TEXT,
				    NULL);
	string = SH_Text_get_string (text, 0, SIZE_MAX, &length, NULL);
	ck_assert_str_eq (string, "<html><body></body></html>");
	free (string);
	SH_Text_free (text);

	text = SH_Fragment_to_html (fragment1, WRAP, 0, 1, INDENT_TEXT,
				    NULL);
	string = SH_Text_get_string (text, 0, SIZE_MAX, &length, NULL);
	ck_assert_str_eq (string,
			  "<html>\n\t<body>\n\t</body>\n</html>\n");
	free (string);
	SH_Text_free (text);

	SH_Fragment_free (fragment1);

	/* error */
	fragment1 = SH_NodeFragment_new ("html", data, NULL);
	fragment2 = SH_NodeFragment_new ("body", data, NULL);
	SH_NodeFragment_append_child (((struct SH_NodeFragment *) fragment1),
				      fragment2, NULL);

	_status_preinit (status);
	text = SH_Fragment_to_html (fragment1, INLINE, 0, 1, INDENT_TEXT,
				    &status);
	string = SH_Text_get_string (text, 0, SIZE_MAX, &length, NULL);
	ck_assert_str_eq (string, "<html><body></body></html>");
	free (string);
	SH_Text_free (text);
	ck_assert_int_eq (status.status, SUCCESS);

	_status_preinit (status);
	text = SH_Fragment_to_html (fragment1, WRAP, 0, 1, INDENT_TEXT,
				    &status);
	string = SH_Text_get_string (text, 0, SIZE_MAX, &length, NULL);
	ck_assert_str_eq (string,
			  "<html>\n\t<body>\n\t</body>\n</html>\n");
	free (string);
	SH_Text_free (text);
	ck_assert_int_eq (status.status, SUCCESS);

	SH_Fragment_free (fragment1);
	SH_Data_free (data);
}
END_TEST

Suite * fragment_suite (void)
{
	Suite *s;
	TCase *tc_core;

	s = suite_create ("Testsuite SeFHT Fragment");

	/* Core test case */
	tc_core = tcase_create ("Core");

	tcase_add_test (tc_core, test_node_fragment);
	tcase_add_test (tc_core, test_node_fragment_raw_new);
	tcase_add_test (tc_core, test_node_fragment_copy);
	tcase_add_test (tc_core, test_node_fragment_deepcopy);
	tcase_add_test (tc_core, test_node_fragment_tag);
	tcase_add_test (tc_core, test_node_fragment_child_alloc);
	tcase_add_test (tc_core, test_node_fragment_child);
	tcase_add_test (tc_core, test_node_fragment_get_child);
	tcase_add_test (tc_core, test_node_fragment_is_child);
	tcase_add_test (tc_core, test_node_fragment_is_descendant);
	tcase_add_test (tc_core, test_node_fragment_is_parent);
	tcase_add_test (tc_core, test_node_fragment_is_ancestor);
	tcase_add_test (tc_core, test_node_fragment_child_insert_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_insert_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_insert_insert_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_insert_insert_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_insert_relative_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_insert_relative_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_remove_self_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_remove_self_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_delete_self_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_delete_self_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_remove_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_remove_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_delete_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_delete_with_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_pop_no_error);
	tcase_add_test (tc_core, test_node_fragment_child_remove_pop_with_error);
	tcase_add_test (tc_core, test_node_fragment_html);
	suite_add_tcase (s, tc_core);

	return s;
}

int main (void)
{
	int number_failed;
	Suite *s;
	SRunner *sr;

	s = fragment_suite ();
	sr = srunner_create (s);

	srunner_run_all (sr, CK_NORMAL);
	number_failed = srunner_ntests_failed (sr);
	srunner_free (sr);

	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

